#include <ctype.h>

#include "hw.h"
#include "f380_reg.h"

#include "bootld.h"
//#include "br_setup.h"
#include "sernoi.h"
#include "print.h"

#pragma codeseg  LOADER_CODE
#pragma constseg LOADER_CONST

__bit f_valid;
__bit code_erased;

void
f380_init(void)
{
    /* WDT disable */
    PCA0MD    &= ~0x40;
    PCA0MD    = 0x00;
    
    /* Port */
    P1MDIN    = 0xDF;
    P0MDOUT   = 0x31;
    P4MDOUT   = 0xF0;
    P1SKIP    = 0x20;
    XBR0      = 0x09;
    XBR1      = 0x40;

    /* Osciccaltor */
    OSCICN= 0xC3;

    /* Timer */
    TMOD= 0x20;
    CKCON= 0x01;
    TH1= 0x64;

    /* Turn ON Vdd monitor */
/*
    VDM0CN= 0x80;
    for (i= 0; i<100; i++) ;
    RSTSRC= 0;
*/
}

void
prompt()
{
    print("Command>");
}

void
print_menu(void)
{
    print(
	"\r\r"
	"C8051F380 Programmer\r"
	"1. Erase and Program\r"
	"2. Cancel\r"
	);
    prompt();
    /*P0=*/ P4= 0xc0;
}

unsigned char
toint(char c)
{
    c= toupper(c);
    if (c<='9')
	return c-'0';
    else
	return c-'A'+10;
}

unsigned char
hex2char()
{
    unsigned char retval;
    char byteH, byteL;
   
    // get a two-byte ASCII representation of a char from the UART
    byteH = serial_receive();/*_getkey();*/
    byteL = serial_receive();/*_getkey();*/
   
    // convert to a single 8 bit result
    retval = (char) toint(byteH) * 16;
    retval += (char) toint(byteL);
    return retval;
}

char
w(__xdata char *p, char d)
{
    __xdata char *pwrite;    	// pointer used for writing FLASH
    __code char *pread;         // pointer used for reading FLASH

    //EA = 0;                   // disable interrupts (precautionary)   
    PFE0CN&= ~FLBWE;		// Single byte write mode
    //FLSCL |= 0x01;            // enable FLASH write/erase
    PSCTL= 0x01;                // MOVX writes FLASH

    pwrite= p;      		// initialize the write pointer
    FLKEY= 0xa5;
    FLKEY= 0xf1;
    *pwrite= d;         	// write one byte to FLASH
    PSCTL= 0x00;                // MOVX writes target XRAM
    //FLSCL &= ~0x01;           // disable FLASH write/erase
    //EA = EA_state;            // restore interrupts to previous state
            
    // verify the checksum
    pread= (__code char *)p;    // initialize the read pointer
    return (*pread==d)?1:0;
}


void
erase_flash(void)
{
    __xdata char *page_pointer;
    int i;

    print("\r*** Erasing 0-0x");print_x(LOADER_START);
    serial_send('\r');
    print("VDM0CN=");print_cx(VDM0CN);print("\r");
    page_pointer= (__xdata char *)0;
    PSCTL= 0x03;                   // MOVX erases FLASH
    for (i= 0; i < LOADER_START/512/*8*(16-2)*/; i++)
    {
	FLKEY= 0xa5;
	FLKEY= 0xf1;
	*page_pointer = 0;         // initiate the erase
	page_pointer+= 512;        // advance to next FLASH page
	if ((i%4) == 0)
	{
	    serial_send('.');
	    //print_c('0'+i/8);
	    if (P4&0x80)
		P4= 0x0f;
	    else
		P4= 0xff;
	}
    }
    PSCTL= 0x00;                    // MOVX writes target XRAM
    P4= 0x0f;

    f_valid= FALSE;                 // indicate that code is no longer valid
    code_erased= TRUE;              // indicate that FLASH has been erased

    w((__xdata char *)(0+0x30), 0x75);
    w((__xdata char *)(1+0x30), 0xd9);
    w((__xdata char *)(2+0x30), 0x00);
    print("\rErased\r");
}

//-----------------------------------------------------------------------------
// receive_code
//-----------------------------------------------------------------------------
// This routine receives the new firmware through the UART in HEX record 
// format.
//
// Hex Record Format:
//
// +--------+--------+------+-------+--------+------(n bytes)------+----------+
// | RECORD | RECLEN |    OFFSET    | RECORD |                     | CHECKSUM |
// |  MARK  |  (n)   |   (2 BYTES)  |  TYPE  |        DATA         |          |
// |  ':'   |        |              |        |                     |          |
// +--------+--------+------+-------+--------+------(n bytes)------+----------+
//
//
void receive_code(void)
{
    __xdata char *pwrite;    			  // pointer used for writing FLASH
    __code char *pread;                    // pointer used for reading FLASH
    unsigned char len;                     // holds the HEX record length field
    unsigned char record_type;             // holds the HEX record type field
    unsigned int offset;                   // holds the HEX record offset field
    // this is the starting address of
    // the code image contained in the 
    // record
    //unsigned int error_in;
    unsigned char checksum;
    unsigned char flash_checksum;
    char c;                                // temporary char
    int i;                                 // temporary int
    int written;
    char code_start[6];
    bit hacked;

    if(!code_erased)
    {
	print("\r*** FLASH not erased\r");
	return;
    }
    else
	print("\rReady to receive...\r");

    f_valid= TRUE;
    code_start[0]= code_start[1]= code_start[2]=
	code_start[3]= code_start[4]= code_start[5]= 0xff;
    //error_in= 0xffff;
    written= 0;
    PFE0CN&= ~FLBWE;		  // Single byte write mode
    do
    {
      
	while ((c= serial_receive()) != ':' );
	hacked= 0;      
	// get the record length
	len= hex2char();
      
	// get the starting address (offset field in HEX record)
	offset= hex2char();                // get the MSB
	offset<<= 8;                       
	offset|= hex2char();               // get the LSB

	// get the record type
	record_type= hex2char();
	if((record_type != 0) && (record_type != 1))
	{
	    print("\r*** Wrong file\r"); 
	    return;
	}      

	pwrite= (__xdata char *)offset;      // initialize the write pointer
	code_erased= FALSE;                // clear the code_erased flag      
           
	PSCTL= 0x01;                      // MOVX writes FLASH
	// write the record into flash
	for(i= 0; i < len; i++)
	{
	    if(pwrite < (__xdata char *)LOADER_START)
	    { 
		unsigned char d;
		d= hex2char();
		/*
		if (pwrite < (__xdata char*)6)
		{
		    code_start[(int)pwrite]= d;
		    hacked= 1;
		}
		else
		*/
		{		
		    FLKEY= 0xa5;
		    FLKEY= 0xf1;
		    *pwrite = d;
		}
		pwrite++;
	    }
	    else
		print("\r\rExceeded space\r");  // print error message

	    written++;
	    if ((written%256) == 0)
	    {
		//serial_send((error_in==0xffff)?'.':'*');
		serial_send('.');
		//P0^= 0x80;
		P4^= 0x80;
	    }
	}
	/*P0=*/ P4= 0x0f;
	PSCTL= 0x00;                       // MOVX writes target XRAM
            
	// verify the checksum
	pread= (__code char *)offset;       // initialize the read pointer
	checksum= hex2char();              // get the HEX record checksum field
	flash_checksum= 0;                 // set the flash_checksum to zero
	// add the data field stored in FLASH to the checksum
	for(i= 0; i < len; i++)
	{
	    flash_checksum+= *pread++;
	} 
      
	// add the remaining fields
	flash_checksum+= len;
	flash_checksum+= (char)(offset >> 8);
	flash_checksum+= (char)(offset & 0x00FF);
	flash_checksum+= record_type;
	flash_checksum+= checksum;
      
	// verify the checksum (the flash_checksum should equal zero)     
	if (!hacked && (flash_checksum != 0))
	{
	    //print("*** Checksum error");
	    //return;
	    //error_in= offset;
	    f_valid= FALSE;
	    //serial_send('*');
	}
	else
	{
	    //serial_send('.');
	}

    }
    while(record_type != 1); 
   
    //w((__xdata char *)3, code_start[0]);
    //w((__xdata char *)4, code_start[1]);
    //w((__xdata char *)5, code_start[2]);

    if (f_valid)
    {
	print("\rOK\r");
    }
    else
    {
	print("\rDownload error"/*" in 0x"*/);
	//print_x(error_in);
	print("\r");
    }

}


void
main(void)
{
    long l= 0;
    //const struct t_board_info *bi;
	
    EA= 0;
    IE= 0;
    IP= 0;
    EIE1= 0;
    EIE2= 0;
    EIP1= 0;
    EIP2= 0;
    //bi= NULL;/*get_board_info();*/
    f380_init(/*bi?(bi->clk_table[bi->used_clk].osc):CLK_22M*/);
    f380_serial_init(/*9600*/);
    //br_init(bi?(bi->used_br):BR_19K);

    //FLSCL= 0x80;
 
    f_valid= 0;
    code_erased= 0;

    while ((P3 & 0xfc) != 0xfc) ;		// Wait to release all buttons
	
    print_menu();
    for (;;)
    {
	if (serial_received())
	{
	    char c= serial_receive();
	    if (c =='1')
	    {
		erase_flash();
		receive_code();
		if (f_valid)
		{
		    print("\r** STARTING APPLICATION **\r\r");
		    RSTSRC= 0x10;
		}
	    }
	    else if (c == '2')
	    {
		print("\r** RESETTING **\r\r");
		RSTSRC= 0x10;      // reset the device   
	    }
	    else
	    {
		print_menu();
	    }
	}
	/*
	  if ((P3 & 0x20) == 0)
	  {
	  br_setup();
	  print_menu();
	  }
	*/
	if (++l > 0x6000)
	{
	    l= 0;
	    //P0^= 0xf0;
	    P4^= 0xf0;
	}
    }
}

/* Local variables: */
/* c-file-style: "linux" */
/* c-basic-offset: 4 */
/* End: */
