//#include <stdio.h>
#include "f020.h"
#include "serial.h"
#include "br_setup.h"
#include "bootld.h"
#include "print.h"

#define M_OK   0x20
#define M_EXIT 0x10
#define M_UP   0x08
#define M_DOWN 0x04

unsigned char p3;

unsigned char
pressed(unsigned mask)
{
  unsigned int i;
  if ((P3&mask) != (p3&mask))
    {
      p3= P3;
      for (i= 0; i<0x8000; i++) ;
      return !(P3&mask);
    }
  return 0;
}

void
pbi(struct t_board_info *bi)
{
	unsigned char i;
  	print_fp("\rBI=%p\r", bi);
	if (bi)
	{
		print("Sign:0x");
		print_lx(bi->magic_sign);
		serial_send('\r');
		for (i= 0; i<NUOF_OSC; i++)
		{
			/*printf("%d. %02x %ld\r", i,
				bi->clk_table[i].osc,
				bi->clk_table[i].freq);*/
			print_f("%d. ", i);
			print_f("%2 ", bi->clk_table[i].osc);
			print_f("%d\r", bi->clk_table[i].freq);
		}
		/*printf("Used:br=%d (%ld) clk=%d freq=%ld\r",
			bi->used_br, br_table[bi->used_br],
			bi->used_clk, bi->clk_table[bi->used_clk].freq);*/
		print_f("Used:br=%d (", bi->used_br);
		print_f("%d) clk=", br_table[bi->used_br]);
		print_f("%d freq=", bi->used_clk);
		print_f("%d\r", bi->clk_table[bi->used_clk].freq);
		/*printf("FREQ:%ld\r", get_Fosc());*/
		print_f("FREQ:%d\r", get_Fosc());
	}
}

void
use_board_info()
{
	const struct t_board_info *bi= get_board_info();
	if (!bi)
		print("\rNo BI\r");
	else
	{
		/*printf_small("Use br=%d osc=%d clk=%ld\r",
			   bi->used_br, bi->used_clk, bi->clk_table[bi->used_clk].freq);*/
		print_f("Use br=%d ", bi->used_br);
		print_f("osc=%d ", bi->used_clk);
		print_f("clk=%d\r", bi->clk_table[bi->used_clk].freq);
		oscillator_init(bi->clk_table[bi->used_clk].osc);
		br_init(bi->used_br);
	}
}

void
dump(unsigned char *p, int nr, int line)
{
	int i;

	for (i= 0; i<nr; i+= line)
	{
		int j;
		print_x((int)p + i);
		print(" ");
		for (j= 0; (j<line) && (i+j < nr); j++)
		{
			print_f("%2 ", p[i+j]);
		}
		print("\r");
	}	
}

void
main(void)
{
  	const struct t_board_info *bi= get_board_info();
	unsigned int i= 0;

  silabs_init(CLK_22M);
  serial_init(9600);
  br_init(BR_19K);
  pbi((struct t_board_info *)bi);
  while (1)
    {
      	reload_code();
      	if (serial_received())
		{
	  		char c= serial_send(serial_receive());
			switch (c)
			{
				case 's':
					save_board_info(0, 0);
					P0= 0xaf;
					break;
				case 'u':
					use_board_info();
					break;
				case 'f':
					save_board_info(0, 1);
					break;
				case 'x':
					dump((XRAM_PTR(unsigned char))0x0e00, PAGE_SIZE, 16);
					break;
				case 'c':
					dump((CODE_PTR(unsigned char))0xfc00, PAGE_SIZE, 16);
					break;
				default:
					pbi(bi);
					break;
			}
		}
      	if (pressed(M_OK))
		{
			br_setup();
			bi= get_board_info();
			pbi(bi);
			//P0= 0x6f;
		}
		if (++i > 0x4000)
		{
			i= 0;
			P0^= 0xf0;
		}
    }
}
