#include "card.h"
/*
/////////////////////////////////////
//  Generated Initialization File  //
/////////////////////////////////////
*/

/*#include "silabs.h"*/

//#pragma codeseg  SILABS_CODE
//#pragma constseg SILABS_CONST


void
Reset_Sources_Init()
{
    WDTCN= 0xDE;
    WDTCN= 0xAD;
}


enum t_br used_br;
unsigned char used_clk;

const long const br_table[]= {
	0, // 0
	110, // 1
	150, // 2
	300, // 3
	600, // 4
	1200, // 5
	2400, // 6
	4800, // 7
	9600, // 8
	19200, // 9
	38400, // 10
	57600, // 11
	115200, // 12
	0
};

const struct t_clk_table const clk_table[NUOF_OSC]= {
	/*  osc  ,   freq  */
	{ CLK_2M ,  2000000 },
	{ CLK_4M ,  4000000 }, //4500000,
	{ CLK_8M ,  8000000 }, //9000000,
	{ CLK_16M, 16000000 }, //18000000,
	{ CLK_11M, 11059200 },
	{ CLK_22M, 22118400 }
};


const struct t_board_info *
get_board_info()
{
	__code struct t_board_info *bi=
		(__code struct t_board_info *)BOARD_INFO;

	if (bi->magic_sign == MAGIC_SIGN)
		return bi;

	for (bi= (__code struct t_board_info *)(0x1000 - RESERVED_SPACE);
		 bi > (__code struct t_board_info *)(0x10000 - RESERVED_SPACE - (2*PAGE_SIZE));
		 bi= (__code struct t_board_info *)(((__code char *)bi) - INFO_BLOCK_SIZE)
		 )
		if (bi->magic_sign == MAGIC_SIGN)
			return bi;

	return 0;
}

long
get_Fosc()
{
	const struct t_clk_table *ctable;
	const struct t_board_info *board_info;

	board_info= get_board_info();
	ctable= (board_info)?(board_info->clk_table):clk_table;

	if ((OSCICN & 0x08)!=0)
	{
		/* external */
		if ((OSCXCN & 0x10)!=0)
			return ctable[4].freq;
		else
			return ctable[5].freq;
	}
	else
	{
		/* internal */
		switch (OSCICN & 7)
		{
			case CLK_2M:  return ctable[0].freq; break;
			case CLK_4M:  return ctable[1].freq; break;
			case CLK_8M:  return ctable[2].freq; break;
			case CLK_16M: return ctable[3].freq; break;
		}
	}
	return 11059200;
}


void
br_init(enum t_br br)
{
	unsigned int r;

	TR2= 0;
	if (br!=BR_0)
	{
		r= 65536-(get_Fosc()/(32*br_table[br]));
 		RCAP2L= r&0xff;
   		RCAP2H= r/256;
		TL2= RCAP2L;
		TH2= RCAP2H;
		TR2= 1;
	}
	used_br= br;
}

void
Port_IO_Init()
{
    P0MDOUT   = 0xF0;
    /*P1MDOUT   = 0xFF;*/
    /*P2MDOUT   = 0xFF;*/
    /*P3MDOUT   = 0xFF;*/
    /*P74OUT    = 0xFF;*/
    XBR0      = 0x04;
    XBR2      = 0x40;
    P0= P1= P2= P3= 0xff;
    P4= P5= P6= P7= 0xff;
}

void
oscillator_init(enum t_osc osc)
{
	__bit EAorg;
    int i;
	
	EAorg= EA;
	EA= 0;
	i = 0;
	if ((P3 & 0x30) == 0)
	{
		if ((osc == CLK_22M) || (osc == CLK_11M))
			osc= CLK_16M;
	}
	if ((osc==CLK_11M) || (osc==CLK_22M))
	{
		/* external */
		OSCXCN= 0x67;
		OSCXCN|= (osc==CLK_11M)?0x10:00;
		OSCICN= (OSCICN&~0x07)|4;
	    for (i= 0; i < 3000; i++) ;  /* Wait 1ms for initialization */
	    for (i= 0; (i<3000) && ((OSCXCN&0x80)!=0x80); i++) ;
		OSCICN= 8|4;
	}
	else
	{
		/* internal */
		OSCICN= (OSCICN&~0x07)|osc;
	    for (i= 0; i < 3000; i++);  /* Wait 1ms for initialization */
	    for (i= 0; (i<3000) && ((OSCICN&0x10)!=0x10); i++) ;
		OSCICN&= ~0x08;
		OSCXCN= 7;
	}
	EA= EAorg;
	for (i= 0; i<NUOF_OSC; i++)
		if (clk_table[i].osc == osc)
			used_clk= i;
}


void
silabs_init(enum t_osc osc)
{
	__code unsigned char *boot_loader= (__code unsigned char *)LOADER_START;
	void (*reprogram)();

	reprogram= (void(*)())LOADER_START;
	if ((P3 & 0x1c) == 0)
		reprogram();

    Port_IO_Init();
    Reset_Sources_Init();
    oscillator_init(osc);
}


/* End of sialbs.c */
/*#include "hw.h"*/

#if defined __SDCC || defined SDCC
#include <stdio.h>
#endif

/*#include "serial.h"*/


volatile unsigned char serial_buffer[SERIAL_BUFFER_SIZE];
volatile unsigned char first_occupied, first_free;
volatile __bit serial_sent;


/* Serial line ISR puts received chars into a ring buffer */

#if defined __SDCC || defined SDCC
void serial_isr(void) __interrupt (4)
#else
interrupt void SCON_int(void)
#endif
{
  if (RI)
    {
      unsigned char new= first_free+1;
      new= new % SERIAL_BUFFER_SIZE;
      if (new != first_occupied)
	{
	  serial_buffer[first_free]= SBUF;
	  first_free= new;
	}
      RI= 0;
    }
  else if (TI)
    {
      serial_sent= 1;
      TI= 0;
    }
}


/* Initialization of serial line */

void
serial_init(long int br)
{
  /* Set  variables */
  first_free= first_occupied= 0;
  if (!(serial_sent= 1))
    serial_dummy();
  
  /* Set USART mode: 8 bit variable speed */
  SCON= 0x40;
  ES= 1;
  REN= 1;

  /* Set Timer1 for 19200 baud, XTAL=11.0592MHz */
  /*TMOD= (TMOD & 0x0f) | 0x20;
  TH1= 0xfd;
  TL1= 0xfd;
  PCON|= 0x80;
  ET1= 0;
  TR1= 1;*/

  /* Set Timer2 as baudrate generator, XTAL=11.0592MHz */
  C_T2= 0;
  CP_RL2= 0;
  {
    long int l= (3*115200)/br;
    TL2= RCAP2L= (65536-l)&0xff; /* 3=115200,6=57600,9=38400,18=19200,36=9600 */
    TH2= RCAP2H= (65536-l)/256;
  }
  RCLK= 1;
  TCLK= 1;
  TR2= 1;

  /* Start */
  EA= 1; /* Enable interrupts */
}


void
beallitas(void)
{
  /* Valtozok beallitasa */
  if (first_occupied= 0)
    serial_dummy();
  first_free= 0;
  serial_sent= 1;
  
  /* Timer1 beallitasa 9600 baud-os sebesseg generalasahoz */
  TMOD= (TMOD & 0x0f) | 0x20;
  TH1= 0xfd;
  TL1= 0xfd;
  PCON&= 0x7f; /* SMOD=0 */
  ET1= 0;
  
  /* Soros vonal beallitasa, 8 bites valtoztathato sebessegu mod */
  SCON= 0x40;
  ES= 1;
  REN= 1;
  
  /* Start */
  TR1= 1;	/* Idozito indul */
  EA= 1;	/* Megszakitasok engedelyezese */
}


/************************************************** Low level line handling */

/* Blocking send */

unsigned char
serial_send(unsigned char c)
{
  while (!serial_sent) ;
  SBUF= c;
  serial_sent= 0;
  return c;
}

int
putchar(int c)
{
  return serial_send(c);
}


/* Check for recived chars */

unsigned char
serial_received(void)
{
  return first_free != first_occupied;
}


/* Blocking receive */

unsigned char
serial_receive(void)
{
  unsigned char c;
  
  while (!serial_received()) ;
  ES= 0;
  c= serial_buffer[first_occupied++];
  first_occupied%= SERIAL_BUFFER_SIZE;
  ES= 1;
  return c;
}

char
getchar(void)
{
  return serial_send(serial_receive());
}


void
serial_dummy(void)
{
  serial_init(0);
  beallitas();
  serial_send(0);
  serial_received();
  putchar(0);
  getchar();
}


/* End of serial.c */
/*#include "hw.h"*/
/*#include "bootld.h"*/


void
reload_code()
{
	__code unsigned char *boot_loader= (__code unsigned char *)LOADER_START;
	void (*reprogram)();

	reprogram= (void(*)())LOADER_START;
	/* 76543210
	   00011100 */
	if (/*(boot_loader[0] == 0xff) &&
		(boot_loader[1] == 0xde) &&
		(boot_loader[2] == 0xff) &&
		(boot_loader[3] == 0xad) &&*/
		((P3 & 0x1c) == 0))
		reprogram();
}


/* End of bootld.c */
#if defined __SDCC || defined SDCC
#include <stdio.h>
#endif

/*#include "print.h"*/

void
print(char *s)
{
  if (s)
    for (; *s; putchar(*s++)) ;
}

void
print_form(char *s, long l, void *p)
{
	if (s)
		for (; *s; s++)
			if (*s != '%')
				putchar(*s);
			else
			{
				s++;
				if (*s)
					switch (*s)
					{
						case 'u': print_u(l); break;
						case 'd': print_d(l); break;
						case 'p': print_p(p); break;
						case 'x': print_x(l); break;
						case '2': print_cx(l); break;
						case '4': print_lx(l); break;
					}
			}
}

/* signed integer in decimal */

void
print_d(long i)
{
  long x= 1000000000;
  char in= 0;
  if (i<0)
    {
      putchar('-');
      i= -i;
    }
  while (x)
    {
      int j= i/x;
      if (in || j || (x==1))
	putchar(j+'0');
      in= in || j;
      i%= x;
      x/= 10;
    }
}


/* unsigned integer in decimal */

void
print_u(unsigned int i)
{
  int x= 10000;
  while (x)
    {
      int j= i/x;
      putchar(j+'0');
      i%= x;
      x/= 10;
    }
}


/* unsigned int in hex */

void
print_cx(unsigned char i)
{
	putchar((i/16)+(((i/16)<10)?'0':('A'-10)));
	putchar((i&15)+(((i&15)<10)?'0':('A'-10)));
}

void
print_x(unsigned int i)
{
/*
  unsigned char j;
  for (j= 0; j<4; j++)
    {
      char v= (i&0xf000)>>12;
      putchar(v+((v<10)?'0':('A'-10)));
      i<<= 4;
    }
*/
	print_cx(i/256);
	print_cx(i&0xff);
}

void
print_lx(unsigned long i)
{
  print_x(i >> 16);
  print_x(i & 0xffff);
}

void
print_p(void *p)
{
	unsigned char t= ((long)p)/0x10000;
	if (t>=0x80)
		putchar('C');
	else if (t>=0x60)
		putchar('P');
	else if (t>=0x40)
		putchar('I');
	else
		putchar('X');
	putchar(':');
	print_x(((long)p)&0xffff);
}


void
print_c(char c)
{
	putchar(c);
}
/*#include "hw.h"*/
/*#include "silabs.h"*/
/*#include "t0.h"*/
/*#include "silprell.h"*/
/*#include "timer.h"*/

volatile __bit btn5, btn4, btn3, btn2;

#if defined SDCC || defined __SDCC
#define BTN5 P3_5
#define BTN4 P3_4
#define BTN3 P3_3
#define BTN2 P3_2
#else
#define BTN5 P3.5
#define BTN4 P3.4
#define BTN3 P3.3
#define BTN2 P3.2
#endif

static volatile unsigned char t0l, t0h;
static volatile unsigned int cnt0= 0;

#if defined __SDCC || defined SDCC
void t0_isr(void) __interrupt (1)
#else
interrupt void T0_int(void)
#endif
{
	TH0= t0h;
	TL0= t0l;
	if (++cnt0 >= DEBOUNCE_TIME)
	{
		btn5= BTN5;
		btn4= BTN4;
		btn3= BTN3;
		btn2= BTN2;
		cnt0= 0;
	}
}

void
debounce(void)
{
	unsigned int i= 0, t0reload;
	Fosc= (float)get_Fosc()/1000000.0;
	t0reload= T0kezdoertek(1.0);
	T0idozito1(0);
	T0beallit(t0reload);
	t0l= t0reload&0xff;
	t0h= t0reload/256;
	btn5= BTN5;
	btn4= BTN4;
	btn3= BTN3;
	btn2= BTN2;
	EA= 1;
	ET0= 1;
	T0start();
}
/*
  t0.c
  Segedlet a T0 idozito/szamlalo kezelesehez
  (c) Drotos Daniel, 2005
*/

/*#include "hw.h"*/
#include <stdio.h>

/*#include "timer.h"*/
/*#include "t0.h"*/



/* Adott idoziteshez szukseges szamlalo kezdoerteket szamolja ki.
   A parametert msec-ben kell megadni (kb 0.02-71 kozott) */

unsigned int T0kezdoertek(float ido /* msec */)
{
  return T_kezdoertek(ido);
}


/* Idozito uzemmod */

void T0idozito1(bool kapuzott)
{
  TR0= 0;
  TMOD&= 0xf0;
  TMOD|= 0x01;
  if (kapuzott)
    TMOD|= 0x08;
}


/* Szamlalo uzemmod */

void T0szamlalo(bool kapuzott)
{
  TR0= 0;
  TMOD&= 0xf0;
  TMOD|= 0x01;
  TMOD|= 0x04;
  if (kapuzott)
    TMOD|= 0x08;
}


/* Szamlalo ertek beallitasa */

void T0beallit(unsigned int kezdoertek)
{
  TH0= kezdoertek >> 8;
  TL0= kezdoertek & 0xff;
}


/* Szamlalo ertek beallitasa */

void T0ujratolt(unsigned int kezdoertek)
{
  TH0= kezdoertek >> 8;
  TL0= kezdoertek & 0xff;
}


/* A szamlalo aktualis allapota usec-ben */

float T0eltelt_us(void)
{
  unsigned char h, l;
  h= TH0;
  l= TL0;
  return (12.0/Fosc) * ((unsigned int)(h*256+l));
}


/* A szamlalo tulcsordulasaig hatralevo ido usec-ben */

float T0hatravan_us(void)
{
  unsigned char h, l;
  h= TH0;
  l= TL0;
  return (12.0/Fosc) * (0x10000 - (unsigned int)(h*256+l));
}


/* End of t0.c */
#ifndef LOADER
//#include <stdio.h>
#endif
/*#include "silabs.h"*/
/*#include "br_setup.h"*/
/*#include "print.h"*/

//#pragma codeseg  BR_SETUP_CODE
//#pragma constseg BR_SETUP_CONST

//__bit b_menu, b_exit, b_up, b_down;

#define menu P3_5
#define exit P3_4
#define up   P3_3
#define down P3_2

/* 76543210
   00MEUD00
*/
#define m_menu 0x20
#define m_ok   m_menu
#define m_exit 0x10
#define m_esc  m_exit
#define m_up   0x08
#define m_down 0x04

#define led7 P0_7
#define led6 P0_6
#define led5 P0_5
#define led4 P0_4

#define LED0 P0_7
#define LED1 P0_6
#define LED2 P0_5
#define LED3 P0_4


static void
debounc()
{
	volatile unsigned int i;
	for (i= get_Fosc()/350; i; i--) ;
}


static unsigned char key_state;

static void
setup_keys()
{
	key_state= P3;
}

static unsigned char
pressed(unsigned char mask)
{
	if ((key_state & mask) != (P3 & mask))
	{
		key_state= (key_state & ~mask) | (P3 & mask);
		debounc();
		return (P3&mask)==0;
	}
	return 0;
}

static void
display3(unsigned char x)
{
	LED0= x&1;
	LED1= x&2;
	LED2= x&4;
}

static void
display4(unsigned char x)
{
	display3(x);
	LED3= x&8;
}


static unsigned char selected_menu;
//static long fosc;

#ifndef LOADER
//extern pbi(struct t_board_info *);
//extern dump(unsigned char *p, int nr, int line);
#endif

void
save_board_info(long freq, unsigned char clear)
{
	__code  struct t_board_info *bi;
	__xdata struct t_board_info *save;
	__xdata char *flash;
	unsigned int i;

	display4(1);
	/* Save last non-reserved page to XRAM */
	bi  = (__code  struct t_board_info *)(BOARD_INFO & ~(PAGE_SIZE-1));
	save= (__xdata struct t_board_info *)(XRAM_SIZE-PAGE_SIZE);

#ifndef LOADER
	print_fp("code %p -> xram ", bi);
	print_fp("%p\r", save);
#endif
	display4(2);
	for (i= 0; i<PAGE_SIZE-INFO_BLOCK_SIZE; i++)
		((__xdata char *)save)[i]= ((__code char *)bi)[i];
	/* Do not copy part of INFO_BLOCK */
	for (; i<PAGE_SIZE; i++)
		((__xdata char *)save)[i]= 0xff;

	bi= (__code  struct t_board_info *)BOARD_INFO;
	if (!clear)
	{
		display4(3);
		save= (__xdata struct t_board_info *)(XRAM_SIZE - INFO_BLOCK_SIZE);

		/* Setup board_info structure in XRAM */
#ifndef LOADER
		print_fp("setup bi %p\r", save);
#endif
		display4(4);
		for (i= 0; i<NUOF_OSC; i++)
		{
			save->clk_table[i].osc= clk_table[i].osc;
			if (bi->magic_sign == MAGIC_SIGN)
				save->clk_table[i].freq=
					((i==used_clk)&&(freq!=0))?freq:bi->clk_table[i].freq;
			else
				save->clk_table[i].freq=
					((i==used_clk)&&(freq!=0))?freq:clk_table[i].freq;
		}
		save->used_br= used_br;
		save->used_clk= used_clk;
		save->magic_sign= MAGIC_SIGN;
#ifndef LOADER
//		pbi(save);
#endif
	}

	display4(5);
	/* Copy XRAM to flash */
	flash= (__xdata char                *)(BOARD_INFO & ~(PAGE_SIZE-1));
	save = (__xdata struct t_board_info *)(XRAM_SIZE-PAGE_SIZE);

#ifndef LOADER
	print_fp("xram %p -> code \r", save);
	print_fp("%p\r", flash);
#endif
	display4(6);
	FLSCL/*|*/= 1;
	PSCTL= 3; /* Erase */
	*flash= 0;
	print_p(flash);
	print_c(' ');
	PSCTL= 1; /* Write */
	display4(7);
	
	for (i= 0; i< PAGE_SIZE; i++)
	{
		/*
		print_x(((int)flash) + i);
		serial_send(' ');
		print_cx(((__xdata char *)save)[i]);
		serial_send('\r');
		*/
		flash[i]= ((__xdata char *)save)[i];
	}

	display4(8);
	PSCTL= 0;
	FLSCL/*&= ~1;*/=0;
	display4(9);
	for (i= 0; i<sizeof(struct t_board_info); i++)
	{
		print_cx(((__code unsigned char *)bi)[i]);
		print_c(' ');
	}
	print_c('\r');
}


static void
selected_osc_changed(unsigned char x)
{
	print_d(x);
	print(": ");
	print_d(clk_table[x].osc);
	print(" ");
	print_d(clk_table[x].freq);
	print("\r");
}

static void
do_select_osc()
{
	enum t_osc x= 0;
	/*printf_small("CLK[%d]=%ld, select OSC...\r",
		used_clk, get_Fosc());*/
	print("CLK["); print_d(used_clk);print("]=");print_d(get_Fosc());print(", select OSC...\r");
	while (1)
	{
		display4(x);
		if (pressed(m_up))
		{
			if (++x>=NUOF_OSC)
				x= 0;
			/*printf_small("%d: %d %ld\r", x, clk_table[x].osc, clk_table[x].freq);*/
			selected_osc_changed(x);
		}
		if (pressed(m_down))
		{
			if (x)
				x= (x-1)&15;
			else
				x= NUOF_OSC-1;
			/*printf_small("%d: %d %ld\r", x, clk_table[x].osc, clk_table[x].freq);*/
			selected_osc_changed(x);
		}
		if (pressed(m_ok))
		{
			/*printf_small("Switching to OSC=%d: %d %ld...\r",
				x, clk_table[x].osc, clk_table[x].freq);*/
			print("Switching to OSC=");
			selected_osc_changed(x);
			oscillator_init(clk_table[x].osc);
			P0^= 0xf0;
			br_init(used_br);
			/*printf_small("Done, clk[%d]=%ld\r", used_clk, get_Fosc());*/
			print("CLK["); print_d(used_clk);print("]=");print_d(get_Fosc());print("\r");
			save_board_info(0, 0);
			//return;
		}
		if (pressed(m_exit))
		{
			return;
		}
	}
}

static void
do_select_br()
{
	enum t_br b= used_br;
	while (1)
	{
		display4(b);
		if (pressed(m_up))
		{
			b= (b+1)&15;
			if (br_table[b]==0)
				b= BR_110;
		}
		if (pressed(m_down))
		{
			b= (b-1)&15;
			if (b<BR_110)
				b= BR_115K;
		}
		if (pressed(m_ok))
		{
			/*printf_small("Changing BR to=%ld (CLK=%ld)...\r",
					br_table[b], get_Fosc());*/
			print("Ch BR=");
			print_d(br_table[b]);
			print(" (CLK=");
			print_d(get_Fosc());
			print(")\r");
			br_init(b);
			P0^= 0xf0;
			/*printf_small("Done, BR=%ld\r", br_table[b]);*/
			print("Done\r");
			save_board_info(0, 0);
		}
		if (pressed(m_exit))
		{
			return;
		}
	}
}

static void
refine_br(long clk)
{
	unsigned int r;
	TR2= 0;
	r= 65536-(clk/(32*br_table[used_br]));
	RCAP2L= r&0xff;
	RCAP2H= r/256;
	TL2= RCAP2L;
	TH2= RCAP2H;
	TR2= 1;
}

static void
do_refine_br()
{
	long clk;
	unsigned char i= 0;

	clk= get_Fosc();
	while (1)
	{
		if (pressed(m_ok))
		{
			save_board_info(clk, 0);
			return;
		}
		if (pressed(m_exit))
		{
			refine_br(get_Fosc());
			return;
		}
		if (pressed(m_up))
		{
			clk+= 100000;
			refine_br(clk);
			i++;
		}
		if (pressed(m_down))
		{
			clk-= 100000;
			refine_br(clk);
			i--;
		}
		display4(i);
		{
			char c;
			/*printf_small("CLK=%ld RCAP2=%x ", clk, RCAP2H*256+RCAP2L);*/
			print("CLK=");
			print_d(clk);
			print(" RCAP2=");
			print_x(RCAP2H*256+RCAP2L);
			print_c(' ');
			for (c='A'; c<'Z'; c++)
				print_c(c);
			print_c('\r');
		}
	}
}

static void
do_sendU()
{
	unsigned char i= 0;
	while (1)
	{
		if (pressed(m_exit) || pressed(m_ok))
			return;
		print_c('U');
		i++;
		display4(i);
	}
}

/*
void
use_board_info()
{
	const struct t_board_info *bi= get_board_info();
	if (!bi)
		print("\rNo BI\r");
	else
	{
		printf_small("Use br=%d osc=%d clk=%ld\r",
			   bi->used_br, bi->used_clk, bi->clk_table[bi->used_clk].freq);
		oscillator_init(bi->clk_table[bi->used_clk].osc);
		br_init(bi->used_br);
	}
}
*/

static const char * const menu_table[]= {
	"Send \"U\"",
	"Set BR",
	"Refine BR",
	"Set OSC",
	"Clear BI",
	"Save BI",
	"",
	"Reset"
};

static void
do_menu()
{
	switch (selected_menu)
	{
		case 0: do_sendU(); break;
		case 1: do_select_br(); break;
		case 2: do_refine_br(); break;
		case 3: do_select_osc(); break;
		case 4: save_board_info(0, 1); break;
		case 5: save_board_info(0, 0); break;
		case 6: break;
		case 7: RSTSRC= 0x10;      // reset the device
				break;
	}
}

static void
print_selected_menu()
{
	print_c(selected_menu+'0');
	print(": ");
	print(menu_table[selected_menu]);
	print_c('\r');
}

void
br_setup()
{
	unsigned int i= 0;
	selected_menu= 0;
	print("\rMenu\r");
	/*printf_small("%c: %s\r", selected_menu+'0', menu_table[selected_menu]);*/
	print_selected_menu();
	while ((P3 & 0xfc) != 0xfc) ;		// Wait to release all buttons
	debounc();
	key_state= P3;
	while (1)
	{
		display3(selected_menu);
		if (pressed(m_menu))
		{
		  print("Starting ");
		  print(menu_table[selected_menu]);
		  print_c('\r');
			do_menu();
			selected_menu= 0;
			P0= 0x0f;
			while ((P3 & 0xfc) != 0xfc) ; // Wait to release all buttons
			return;
		}
		if (pressed(m_exit))
		{
			selected_menu= 0;
			P0= 0x0f;
			print("Exit\r");
			while ((P3 & 0xfc) != 0xfc) ; // Wait to release all buttons
			return;
		}
		if (pressed(m_up))
		{
			selected_menu= (selected_menu+1) & 7;
			/*printf_small("%c: %s\r", selected_menu+'0',
					menu_table[selected_menu]);*/
			print_selected_menu();
		}
		if (pressed(m_down))
		{
			selected_menu= (selected_menu-1) & 7;
			/*printf_small("%c: %s\r", selected_menu+'0',
					menu_table[selected_menu]);*/
			print_selected_menu();
		}
		if (i++ > 0x4000)
		{
			i= 0;
			LED3= !LED3;
		}
	}
}


/* End of br_setup.c */
/*#include "timer.h"*/


float Fosc= 11.0592; /* MHz, Oszcillator orajel */


/* Adott idoziteshez szukseges szamlalo kezdoerteket szamolja ki.
   A parametert msec-ben kell megadni (kb 0.02-71 kozott) */

unsigned int T_kezdoertek(float ido /* msec */)
{
  float tper;	/* szamlalo periodusideje */
  //float per;	/* szukseges periodusok szama */

  tper= 12.0/Fosc;
  /* atvaltas usec-be */
  ido*= 1000.0;
  /* Max 65529 lepes */
  if (ido > tper*65529.0)
    return 0;
  /* Min 20 lepes */
  if (ido < tper*20.0)
    return 0xffff-20;
  //per= ido/tper;
  return (unsigned int)(65541.0-ido/tper);
}

/*#include "silabs.h"*/
/*#include "serial.h"*/
/*#include "bootld.h"*/


unsigned char
silabs_receive()
{
  while (!serial_received())
  	reload_code();
  return serial_receive();
}


/* End of silser.c */


void
card_init(void)
{
  silabs_init(CLK_22M);
  serial_init(9600);
  br_init(BR_19K);
  debounce();
  P0= 0x00;
}

unsigned char
get_button5(void)
{
  return btn5?0:1;
}

unsigned char
get_button4(void)
{
  return btn4?0:1;
}

unsigned char
get_button3(void)
{
  return btn3?0:1;
}

unsigned char
get_button2(void)
{
  return btn2?0:1;
}

void
set_led4(unsigned char ON)
{
  led4= ON?1:0;
}

void
set_led5(unsigned char ON)
{
  led5= ON?1:0;
}

void
set_led6(unsigned char ON)
{
  led6= ON?1:0;
}

void
set_led7(unsigned char ON)
{
  led7= ON?1:0;
}
