#include <stdio.h>

#include "f380.h"
#include "bootld.h"
#include "serial.h"
#include "silser.h"
//#include "analog.h"
#include "timer.h"
#include "t0.h"

#include "card2014.h"


/* Debounce buttons */
volatile int cnt_debounce= 0;
volatile int time_debounce= 500;
volatile bit btn_menu, btn_exit, btn_up, btn_down;


/* Handle LEDs */
volatile signed char led_powers[4]= { 100, 100, 100, 100 };

volatile signed char cnt_led= 0;


/* Timer interrupt */
volatile unsigned char t0l, t0h;
volatile int sw_counter= 0;
volatile unsigned int dpre= 0;
volatile unsigned char dcnt= 0;
volatile unsigned char rmask= LED4;

#if defined __SDCC || defined SDCC
void t0_isr(void) __interrupt (1)
#else
interrupt void T0_int(void)
#endif
{
  TH0= t0h;
  TL0= t0l;
  /* Debounce buttons */
  if (++cnt_debounce >= time_debounce)
    {
      cnt_debounce= 0;
      btn_menu= BTN5;
      btn_exit= BTN4;
      btn_up= BTN3;
      btn_down= BTN2;
    }
  /* handle LEDs */
  if (++cnt_led > 99)
    cnt_led= 0;
  LED= rmask &
    (
     (LED & ~LED_MASK) |
     ((cnt_led < led_powers[0])?LED4:0) |
     ((cnt_led < led_powers[1])?LED5:0) |
     ((cnt_led < led_powers[2])?LED6:0) |
     ((cnt_led < led_powers[3])?LED7:0)
     );
  /* General counter */
  if (++sw_counter >= 10000)
    sw_counter= 0;
  /* 0.1 sec pre-counter */
  if (++dpre >= 100)
    {
      dpre= 0;
      if (dcnt)
	dcnt--;
    }
}


/* Handle button presses */
bit prev_menu, prev_exit, prev_up, prev_down;
bit press_menu= 0, press_exit= 0, press_up= 0, press_down= 0;

void
check_buttons()
{
  if (btn_menu != prev_menu)
    {
      prev_menu= btn_menu;
      if (!btn_menu)
	press_menu= 1;
    }
  if (btn_exit != prev_exit)
    {
      prev_exit= btn_exit;
      if (!btn_exit)
	press_exit= 1;
    }
  if (btn_up != prev_up)
    {
      prev_up= btn_up;
      if (!btn_up)
	press_up= 1;
    }
  if (btn_down != prev_down)
    {
      prev_down= btn_down;
      if (!btn_down)
	press_down= 1;
    }
}

unsigned char
pressed_menu()
{
  if (!press_menu)
    return 0;
  press_menu= 0;
  return 1;
}

unsigned char
pressed_exit()
{
  if (!press_exit)
    return 0;
  press_exit= 0;
  return 1;
}

unsigned char
pressed_up()
{
  if (!press_up)
    return 0;
  press_up= 0;
  return 1;
}

unsigned char
pressed_down()
{
  if (!press_down)
    return 0;
  press_down= 0;
  return 1;
}

void
chk_ud()
{
  if (pressed_up())
    {
      if ((led_powers[0]+= 10) > 100)
	led_powers[0]= -1;
      printf("LED power increased to %d\n", led_powers[0]);
    }
  if (pressed_down())
    {
      if ((led_powers[0]-= 10) < 0)
	led_powers[0]= 100;
      printf("LED power decreased to %d\n", led_powers[0]);
    }  
  led_powers[1]= led_powers[0];
  led_powers[2]= led_powers[0];
  led_powers[3]= led_powers[0];
}

/* Demonstrations */

unsigned char rspeed= 20;

void
demo0(void)
{
  if (dcnt == 0)
    {
      //printf("shift %02x\n", rmask);
      dcnt= rspeed;
      rmask<<= 1;
      if (rmask == 0)
	rmask= LED4;
    }
  if (pressed_exit())
    {
      rspeed-= 10;
      if (rspeed < 10)
	rspeed= 100;
    }
  chk_ud();
}

unsigned char d1led= 0;

void
demo1()
{
  rmask= LED4 << d1led;
  if (dcnt == 0)
    {
      dcnt= 10;
      if ((led_powers[d1led]+= 10) > 99)
	{
	  led_powers[d1led]= 0;
	  if (++d1led > 3)
	    d1led= 0;
	  led_powers[d1led]= 0;
	}
    }
}

void
demo2(void)
{
  chk_ud();
}

/*
void
demo1(void)
{
  led_powers[0]= analog_p();
  led_powers[1]= led_powers[0];
  led_powers[2]= led_powers[0];
  led_powers[3]= led_powers[0];
}
*/
int nuof_demo= 3;

/* Main program */
void
main(void)
{
  unsigned int t0reload;
  int demo= 0;

  f380_init();
  f380_serial_init();
  //br_init(BR_19K);
  //analog_init(2);
  printf("RSTSRC=%02x\n", rst_reason);
  
  prev_menu= btn_menu= BTN5;
  prev_exit= btn_exit= BTN4;
  prev_up= btn_up= BTN3;
  prev_down= btn_down= BTN2;

  Fosc= (float)get_Fosc()/1000000.0;
  t0reload= T_kezdoertek(0.1);
  T0idozito1(0);
  T0beallit(t0reload);
  t0l= t0reload&0xff;
  t0h= t0reload/256;
  EA= 1;
  ET0= 1;
  T0start();
  
  for (;;)
    {
      reload_code();
      check_buttons();
      switch (demo)
	{
	case 0: demo0(); break;
	case 1: demo1(); break;
	case 2: demo2(); break;
	}
      if (pressed_menu())
	{
	  demo++;
	  led_powers[0]= 100;
	  led_powers[1]= 100;
	  led_powers[2]= 100;
	  led_powers[3]= 100;
	  if (demo >= nuof_demo)
	    demo= 0;
	  switch (demo)
	    {
	    case 0:
	      rmask= LED4;
	      break;
	    case 1:
	      rmask= LED4;
	      d1led= 0;
	      led_powers[0]= 0;
	      led_powers[1]= 0;
	      led_powers[2]= 0;
	      led_powers[3]= 0;  
	      break;
	    case 2:
	      rmask= 0xff;
	      break;
	    }
	  printf("Starting demo %d\n", demo);
	}
    }
}
